@@ -42,6 +42,50 @@ def direct_tcpip(client, host, port):
     )
 
 
+def get_gateway(host, port, cache, replace=False):
+    """
+    Create and return a gateway socket, if one is needed.
+
+    This function checks ``env`` for gateway or proxy-command settings and
+    returns the necessary socket-like object for use by a final host
+    connection.
+
+    :param host:
+        Hostname of target server.
+
+    :param port:
+        Port to connect to on target server.
+
+    :param cache:
+        A ``HostConnectionCache`` object, in which gateway ``SSHClient``
+        objects are to be retrieved/cached.
+
+    :param replace:
+        Whether to forcibly replace a cached gateway client object.
+
+    :returns:
+        A ``socket.socket``-like object, or ``None`` if none was created.
+    """
+    from fabric.state import env, output
+    sock = None
+    proxy_command = ssh_config().get('proxycommand', None)
+    if env.gateway:
+        gateway = normalize_to_string(env.gateway)
+        # ensure initial gateway connection
+        if replace or gateway not in cache:
+            if output.debug:
+                print "Creating new gateway connection to %r" % gateway
+            cache[gateway] = connect(*normalize(gateway) + (cache, False))
+        # now we should have an open gw connection and can ask it for a
+        # direct-tcpip channel to the real target. (bypass cache's own
+        # __getitem__ override to avoid hilarity - this is usually called
+        # within that method.)
+        sock = direct_tcpip(dict.__getitem__(cache, gateway), host, port)
+    elif proxy_command:
+        sock = ssh.proxycommand(proxy_command)
+    return sock
+
+
 class HostConnectionCache(dict):
     """
     Dict subclass allowing for caching of host connections/clients.
@@ -80,25 +124,9 @@ class HostConnectionCache(dict):
         """
         Force a new connection to ``key`` host string.
         """
-        from fabric.state import env, output
         user, host, port = normalize(key)
         key = normalize_to_string(key)
-        sock = None
-        proxy_command = ssh_config().get('proxycommand', None)
-        if env.gateway:
-            gateway = normalize_to_string(env.gateway)
-            # Ensure initial gateway connection
-            if gateway not in self:
-                if output.debug:
-                    print "Creating new gateway connection to %r" % gateway
-                self[gateway] = connect(*normalize(gateway))
-            # Now we should have an open gw connection and can ask it for a
-            # direct-tcpip channel to the real target. (Bypass our own
-            # __getitem__ override to avoid hilarity.)
-            sock = direct_tcpip(dict.__getitem__(self, gateway), host, port)
-        elif proxy_command:
-            sock = ssh.ProxyCommand(proxy_command)
-        self[key] = connect(user, host, port, sock)
+        self[key] = connect(user, host, port, cache=self)
 
     def __getitem__(self, key):
         """
@@ -302,12 +330,23 @@ def normalize_to_string(host_string):
     return join_host_strings(*normalize(host_string))
 
 
-def connect(user, host, port, sock=None):
+def connect(user, host, port, cache, seek_gateway=True):
     """
     Create and return a new SSHClient instance connected to given host.
 
-    If ``sock`` is given, it's passed into ``SSHClient.connect()`` directly.
-    Used for gateway connections by e.g. ``HostConnectionCache``.
+    :param user: Username to connect as.
+
+    :param host: Network hostname.
+
+    :param port: SSH daemon port.
+
+    :param cache:
+        A ``HostConnectionCache`` instance used to cache/store gateway hosts
+        when gatewaying is enabled.
+
+    :param seek_gateway:
+        Whether to try setting up a gateway socket for this connection. Used so
+        the actual gateway connection can prevent recursion.
     """
     from state import env, output
 
@@ -333,12 +372,20 @@ def connect(user, host, port, sock=None):
     connected = False
     password = get_password()
     tries = 0
+    sock = None
 
     # Loop until successful connect (keep prompting for new password)
     while not connected:
         # Attempt connection
         try:
             tries += 1
+
+            # (Re)connect gateway socket, if needed.
+            # Nuke cached client object if not on initial try.
+            if seek_gateway:
+                sock = get_gateway(host, port, cache, replace=tries > 0)
+
+            # Ready to connect
             client.connect(
                 hostname=host,
                 port=int(port),
