@@ -37,12 +37,14 @@ class Task(object):
     is_default = False
 
     # TODO: make it so that this wraps other decorators as expected
-    def __init__(self, alias=None, aliases=None, default=False,
+    def __init__(self, alias=None, aliases=None, default=False, name=None,
         *args, **kwargs):
         if alias is not None:
             self.aliases = [alias, ]
         if aliases is not None:
             self.aliases = aliases
+        if name is not None:
+            self.name = name
         self.is_default = default
 
     def run(self):
@@ -78,12 +80,14 @@ class Task(object):
         # change)
         default_pool_size = default or len(hosts)
         # Allow per-task override
-        pool_size = getattr(self, 'pool_size', default_pool_size)
+        # Also cast to int in case somebody gave a string
+        from_task = getattr(self, 'pool_size', None)
+        pool_size = int(from_task or default_pool_size)
         # But ensure it's never larger than the number of hosts
         pool_size = min((pool_size, len(hosts)))
         # Inform user of final pool size for this task
         if state.output.debug:
-            print "Parallel tasks now using pool size of %d" % pool_size
+            print("Parallel tasks now using pool size of %d" % pool_size)
         return pool_size
 
 
@@ -91,9 +95,11 @@ class WrappedCallableTask(Task):
     """
     Wraps a given callable transparently, while marking it as a valid Task.
 
-    Generally used via `@task <~fabric.decorators.task>` and not directly.
+    Generally used via `~fabric.decorators.task` and not directly.
 
     .. versionadded:: 1.1
+
+    .. seealso:: `~fabric.docs.unwrap_tasks`, `~fabric.decorators.task`
     """
     def __init__(self, callable, *args, **kwargs):
         super(WrappedCallableTask, self).__init__(*args, **kwargs)
@@ -101,7 +107,10 @@ class WrappedCallableTask(Task):
         # Don't use getattr() here -- we want to avoid touching self.name
         # entirely so the superclass' value remains default.
         if hasattr(callable, '__name__'):
-            self.__name__ = self.name = callable.__name__
+            if self.name == 'undefined':
+                self.__name__ = self.name = callable.__name__
+            else:
+                self.__name__ = self.name
         if hasattr(callable, '__doc__'):
             self.__doc__ = callable.__doc__
         if hasattr(callable, '__module__'):
@@ -178,7 +187,7 @@ def _execute(task, host, my_env, args, kwargs, jobs, queue, multiprocessing):
                 # clear what host encountered the exception that will
                 # print.
                 if e.__class__ is not SystemExit:
-                    print >> sys.stderr, "!!! Parallel execution exception under host %r:" % name
+                    sys.stderr.write("!!! Parallel execution exception under host %r:\n" % name)
                     submit(e)
                 # Here, anything -- unexpected exceptions, or abort()
                 # driven SystemExits -- will bubble up and terminate the
@@ -226,8 +235,10 @@ def execute(task, *args, **kwargs):
     taskname:host=hostname``.
 
     Any other arguments or keyword arguments will be passed verbatim into
-    ``task`` when it is called, so ``execute(mytask, 'arg1', kwarg1='value')``
-    will (once per host) invoke ``mytask('arg1', kwarg1='value')``.
+    ``task`` (the function itself -- not the ``@task`` decorator wrapping your
+    function!) when it is called, so ``execute(mytask, 'arg1',
+    kwarg1='value')`` will (once per host) invoke ``mytask('arg1',
+    kwarg1='value')``.
 
     This function returns a dictionary mapping host strings to the given task's
     return value for that host's execution run. For example, ``execute(foo,
