@@ -5,20 +5,20 @@ import copy
 import getpass
 import sys
 
-import ssh
 from nose.tools import with_setup, ok_, raises
 from fudge import (Fake, clear_calls, clear_expectations, patch_object, verify,
     with_patched_object, patched_context, with_fakes)
 
 from fabric.context_managers import settings, hide, show
 from fabric.network import (HostConnectionCache, join_host_strings, normalize,
-    denormalize, key_filenames)
+    denormalize, key_filenames, ssh)
 from fabric.io import output_loop
 import fabric.network  # So I can call patch_object correctly. Sigh.
 from fabric.state import env, output, _get_system_username
 from fabric.operations import run, sudo, prompt
 from fabric.exceptions import NetworkError
 from fabric.tasks import execute
+from fabric import utils # for patching
 
 from utils import *
 from server import (server, PORT, RESPONSES, PASSWORDS, CLIENT_PRIVKEY, USER,
@@ -47,6 +47,29 @@ class TestNetwork(FabricTest):
             yield eq_, normalize(input), normalize(output_)
             del eq_.description
 
+    def test_normalization_for_ipv6(self):
+        """
+        normalize() will accept IPv6 notation and can separate host and port
+        """
+        username = _get_system_username()
+        for description, input, output_ in (
+            ("Full IPv6 address",
+                '2001:DB8:0:0:0:0:0:1', (username, '2001:DB8:0:0:0:0:0:1', '22')),
+            ("IPv6 address in short form",
+                '2001:DB8::1', (username, '2001:DB8::1', '22')),
+            ("IPv6 localhost",
+                '::1', (username, '::1', '22')),
+            ("Square brackets are required to separate non-standard port from IPv6 address",
+                '[2001:DB8::1]:1222', (username, '2001:DB8::1', '1222')),
+            ("Username and IPv6 address",
+                'user@2001:DB8::1', ('user', '2001:DB8::1', '22')),
+            ("Username and IPv6 address with non-standard port",
+                'user@[2001:DB8::1]:1222', ('user', '2001:DB8::1', '1222')),
+        ):
+            eq_.description = "Host-string IPv6 normalization: %s" % description
+            yield eq_, normalize(input), output_
+            del eq_.description
+
     def test_normalization_without_port(self):
         """
         normalize() and join_host_strings() omit port if omit_port given
@@ -56,6 +79,23 @@ class TestNetwork(FabricTest):
             'user@localhost'
         )
 
+    def test_ipv6_host_strings_join(self):
+        """
+        join_host_strings() should use square brackets only for IPv6 and if port is given
+        """
+        eq_(
+            join_host_strings('user', '2001:DB8::1'),
+            'user@2001:DB8::1'
+        )
+        eq_(
+            join_host_strings('user', '2001:DB8::1', '1222'),
+            'user@[2001:DB8::1]:1222'
+        )
+        eq_(
+            join_host_strings('user', '192.168.0.0', '1222'),
+            'user@192.168.0.0:1222'
+        )
+
     def test_nonword_character_in_username(self):
         """
         normalize() will accept non-word characters in the username part
@@ -95,6 +135,8 @@ class TestNetwork(FabricTest):
                 'user@localhost', 'user@localhost:22'),
             ("Both username and port",
                 'localhost', username + '@localhost:22'),
+            ("IPv6 address",
+                '2001:DB8::1', username + '@[2001:DB8::1]:22'),
         ):
             eq_.description = "Host-string denormalization: %s" % description
             yield eq_, denormalize(string1), denormalize(string2)
@@ -377,7 +419,7 @@ class TestNetwork(FabricTest):
 [%(prefix)s] out: result1
 [%(prefix)s] out: result2
 """ % {'prefix': env.host_string, 'user': env.user}
-        eq_(expected[1:], sys.stdall.getvalue())
+        eq_(sys.stdall.getvalue(), expected[1:])
 
     @mock_streams('both')
     @server(pubkeys=True, responses={'silent': '', 'normal': 'foo'})
@@ -573,8 +615,9 @@ class TestSSHConfig(FabricTest):
         eq_(normalize("localhost")[1], "localhost")
         eq_(normalize("myalias")[1], "otherhost")
 
-    @aborts
-    def test_aborts_with_bad_config_file_path(self):
+    @with_patched_object(utils, 'warn', Fake('warn', callable=True,
+        expect_call=True))
+    def test_warns_with_bad_config_file_path(self):
         # use_ssh_config is already set in our env_setup()
         with settings(ssh_config_path="nope_bad_lol"):
             normalize('foo')
@@ -632,14 +675,3 @@ class TestKeyFilenames(FabricTest):
             with settings(key_filename=["bizbaz.pub", "whatever.pub"]):
                 expected = ["bizbaz.pub", "whatever.pub", "foobar.pub"]
                 eq_(key_filenames(), expected)
-
-    def test_specific_host(self):
-        """
-        SSH lookup aspect should correctly select per-host value
-        """
-        with settings(
-            use_ssh_config=True,
-            ssh_config_path=support("ssh_config"),
-            host_string="myhost"
-        ):
-            eq_(key_filenames(), ["neighbor.pub"])
